/*****************************************************************************
 Copyright (c) 2004, Intel Corporation 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.

 3. Neither the name of Intel Corporation nor the names of its contributors 
 may be used to endorse or promote products derived from this software 
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

****************************************************************************/

#include"softcore.h"
#include"hamcore.h"

#include"softserial.h"

extern struct global G;

//=============================================================================
void softserial_put_char(struct tty_struct* ptty,
                        unsigned char ch)
{
  // printk("softserial:softserial_put_char()\n");
   G.softcore.putchar(ch);
}

//=============================================================================
void softserial_flush_chars(struct tty_struct* ptty)
{
  // printk("softserial:softserial_flush_chars()\n");
   //does nothing
}

//=============================================================================
static DECLARE_MUTEX(softserial_write_sem);
int softserial_write(struct tty_struct* ptty,
                      int from_user_space,
                      const unsigned char* input_buffer,
                      int write_count_asked)  //why is it a signed int?
{

   int written_count  = 0;
   int softcore_space = 0;
   int copy_size = 0;
   const unsigned char* input_buffer_ptr = input_buffer;
   
//   return(write_count_asked);
   
   if(write_count_asked <=0) return (-EFAULT);


   //printk("softserial:softserial_write()\n");
   //printk("write: fus%d, count%d\n",from_user_space, write_count_asked);
   
   if(from_user_space)
   {
      down(&softserial_write_sem); //this may not be enough
      while(write_count_asked - written_count > 0)
      {
         softcore_space = G.softcore.write_free();
         if(softcore_space == 0) break;  //try again?  potential infinite loop?
         //printk("space free %d\n",softcore_space);
	 
         if(write_count_asked > softcore_space)
         {
            copy_size = softcore_space;
         }
         else
         {
            copy_size = write_count_asked;
         }

         copy_from_user(G.softcore.tx_fifo, input_buffer_ptr, copy_size);
         //printk("tx_fifo[0]=%c copysize=%d",*(G.softcore.tx_fifo),copy_size);
	// if(copy_size <= 0)
        // {
        //    return(-EFAULT);
        // }
	

         G.softcore.write(copy_size);

         input_buffer_ptr += copy_size;
         written_count += copy_size;
      }
      up(&softserial_write_sem);
   }
   else           //data is from kernel space
   {
      while(write_count_asked - written_count > 0)
      {
         softcore_space = G.softcore.write_free();
         if(softcore_space == 0) break;  //try again?  potential infinite loop?
         if(write_count_asked > softcore_space)
         {
            copy_size = softcore_space;
         }
         else
         {
            copy_size = write_count_asked;
         }

         memcpy(G.softcore.tx_fifo, input_buffer_ptr, copy_size);
         G.softcore.write(copy_size);

         input_buffer_ptr += copy_size;
         written_count += copy_size;
      }
   }

   return(written_count);
}

//=============================================================================
int softserial_write_room(struct tty_struct* ptty)
{
   //printk("softserial:softserial_write_room()\n");
   return(G.softcore.write_free());
}

//=============================================================================
int softserial_chars_in_buffer(struct tty_struct* ptty)
{
  // printk("softserial:softserial_chars_in_buffer()%d\n",
//		   G.softcore.write_pending());
   return(G.softcore.write_pending());
}

//=============================================================================
void softserial_flush_buffer(struct tty_struct* ptty)
{
   //printk("softserial:softserial_flush_buffer()\n");
   G.softcore.write_flush();
}

//=============================================================================
void softserial_wait_until_sent(struct tty_struct* ptty,
                                int jiffies_waittime)
{
   unsigned long start = jiffies;
   //printk("softserial:softserial_wait_until_sent()\n");

   while(1)
   {
      if(G.softcore.write_pending() == 0) return;
      current->state = TASK_INTERRUPTIBLE;
      schedule_timeout(1);
      if(signal_pending(current)) break;
      if(jiffies_waittime && (jiffies - start >= jiffies_waittime)) break;
   }
   current->state = TASK_RUNNING;
}

//=============================================================================
void softserial_throttle(struct tty_struct* ptty)
{
  // printk("softserial:softserial_throttle()\n");
   if(I_IXOFF(ptty)) G.softcore.send_xchar(STOP_CHAR(ptty));
   G.softcore.rts_off();
}


//=============================================================================
void softserial_unthrottle(struct tty_struct* ptty)
{
   //printk("softserial:softserial_unthrottle()\n");
   G.softcore.rts_off();
   G.softcore.send_xchar(START_CHAR(ptty));
}

//=============================================================================
void softserial_send_xchar(struct tty_struct* ptty,
                           char softchar)
{
  // printk("softserial:softserial_send_xchar()\n");
   G.softcore.send_xchar(softchar);
}

//=============================================================================
void softserial_set_termios(struct tty_struct* ptty,
                            struct termios* previous_termios)
{
struct async_struct* ptr_async_struct;
//   printk("softserial:softserial_set_termios()\n");

   if(!ptty)
   {
	   printk("softserial_set_termios:set termios  null ptty\n");
	   return;
   }
   if(!(ptty->driver_data))
   {
	   printk("softserial_set_termios:set termios null driver data\n");
	   return;
   }
   
ptr_async_struct = (struct async_struct*)(ptty->driver_data);
   if(ptty->termios->c_cflag & CRTSCTS)
	   ptr_async_struct->flags |=  ASYNC_CTS_FLOW;
   else    
	   ptr_async_struct->flags &= ~ASYNC_CTS_FLOW;

   if(ptty->termios->c_cflag & CLOCAL)  
	   ptr_async_struct->flags &= ~ASYNC_CHECK_CD;
   else    
	   ptr_async_struct->flags |=  ASYNC_CHECK_CD;


}

//=============================================================================
void softserial_break(struct tty_struct* ptty,
                      int state)
{
   //printk("softserial:softserial_break()\n");
}
